@@ -77,20 +77,20 @@ goana.MArrayLM <- function(de, coef = ncol(de), geneid = rownames(de), FDR = 0.0
 goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL, covariate=NULL, plot=FALSE, ...)
 #	Gene ontology analysis of DE genes
 #	Gordon Smyth and Yifang Hu
-#	Created 20 June 2014.  Last modified 22 Feb 2018.
+#	Created 20 June 2014.  Last modified 19 May 2019.
 {
 #	Get access to package of GO terms
 	suppressPackageStartupMessages(OK <- requireNamespace("GO.db",quietly=TRUE))
-	if(!OK) stop("GO.db package required but not installed (or can't be loaded)")
+	if(!OK) stop("GO.db package required but is not installed (or can't be loaded)")
 
 #	Get access to required annotation functions
 	suppressPackageStartupMessages(OK <- requireNamespace("AnnotationDbi",quietly=TRUE))
-	if(!OK) stop("AnnotationDbi package required but not installed (or can't be loaded)")
+	if(!OK) stop("AnnotationDbi package required but is not installed (or can't be loaded)")
 
 #	Load appropriate organism package
 	orgPkg <- paste0("org.",species,".eg.db")
 	suppressPackageStartupMessages(OK <- requireNamespace(orgPkg,quietly=TRUE))
-	if(!OK) stop(orgPkg," package required but not not installed (or can't be loaded)")
+	if(!OK) stop(orgPkg," package required but is not installed (or can't be loaded)")
 
 #	Get GO to Entrez Gene mappings
 	obj <- paste0("org.",species,".egGO2ALLEGS")
@@ -170,7 +170,7 @@ goana.default <- function(de, universe = NULL, species = "Hs", prior.prob = NULL
 		isDE <- (universe %in% unlist(de))
 		o <- order(covariate)
 		prior.prob <- covariate
-		span <- approx(x=c(20,200),y=c(1,0.5),xout=sum(isDE),rule=2)$y
+		span <- approx(x=c(20,200),y=c(1,0.5),xout=sum(isDE),rule=2,ties=list("ordered",mean))$y
 		prior.prob[o] <- tricubeMovingAverage(isDE[o],span=span)
 		if(plot) barcodeplot(covariate, index=isDE, worm=TRUE, span.worm=span, main="DE status vs covariate")
 	}
