@@ -157,3 +157,17 @@ function(object, i, j)
 	object
 })
 
+assign("[.TestResults",
+function(object, i, j)
+#  Subsetting for TestResults objects
+#  Gordon Smyth
+#  16 June 2019. Last modified.
+{
+	if(nargs() == 1L) return(object)
+
+	if(nargs() == 2L) return(object@.Data[i])
+
+	object@.Data <- object@.Data[i,j,drop=FALSE]
+	object
+})
+
