@@ -31,7 +31,7 @@ zscoreGamma <- function(q, shape, rate = 1, scale = 1/rate)
 	z
 }
 
-zscoreT <- function(x, df, approx=FALSE, method="hill")
+zscoreT <- function(x, df, approx=FALSE, method="bailey")
 #  Z-score equivalents of t distribution deviates
 #  Gordon Smyth
 #  Created 24 August 2003 with exact quantile method only.
@@ -90,10 +90,10 @@ zscoreT <- function(x, df, approx=FALSE, method="hill")
 	z * sign(x)
 }
 
-tZscore <- function(x, df)
+tZscore <- function(z, df)
 #  t-statistic equivalents of z-score deviates
 #  Gordon Smyth
 #  Created 1 June 2004. Modified 21 July 2019.
 {
-	qt(pnorm(abs(x),lower.tail=FALSE,log.p=TRUE),df=df,lower.tail=FALSE,log.p=TRUE) * sign(x)
+	qt(pnorm(abs(z),lower.tail=FALSE,log.p=TRUE),df=df,lower.tail=FALSE,log.p=TRUE) * sign(z)
 }
